''
'' CVC Player for FreeBasic
'' by ssjx (2011 - 2020)
''
'' Add info/credits at the bottom - [R]eplay [Q]uit [P]ause
''
'' 13/2/20 	- Can now handle lower res videos
''			- More checks added
''
'' 28/1/17 	- Added some resolution safety checks and other nicities
'' 

#include "cvc.bi"
#define TITLE "CVC Player"
#define VERSION "v0.3"
#define SITEURL "http://ssjx.co.uk"

#define SC_ESCAPE &h01

declare sub dec_enc_cv1( as byte ptr, as byte ptr, as integer)
declare sub dec1_cv1( as byte ptr,  as byte ptr, as integer)
declare sub dec2_cv1( as byte ptr,  as byte ptr, as integer)

dim shared cycolor(4) as uinteger

cycolor(3)=rgb(0,0,0)
cycolor(2)=rgb(96,96,96)
cycolor(1)=rgb(192,192,192)
cycolor(0)=rgb(255,255,255)

dim header as cvc_header
dim vid_header as cvc_video
dim frm_header as cvc_frame

const as integer ofx=(640-320)/2
const as integer ofy=(480-200)/2

''
''
''

function swapshort(in as ushort) as integer
	dim as ushort value
	value =(in and &hff00) shr 8 + (in and &hff) shl 8
	return cint(value)
end function

function swaplong(in as ulong) as integer
	dim as ulong value
	value =(in and &hff000000) shr 24 + (in and &h00ff0000) shr 8+(in and &hff) shl 24+(in and &hff00) shl 8
	return value
end function

sub memcpy(outa as byte ptr,ina as byte ptr,size as integer)
	dim i as integer
	for i=0 to size-1
		*(outa+i)=*(ina+i)
	next i
end sub

sub memclr(ina as byte ptr,size as integer)
	dim i as integer
	for i=0 to size-1
		*(ina+i)=0
	next i
end sub


sub block(x as integer,y as integer,col as uinteger)
	x=x shl 1 '' *2
	y=y shl 1 '' *2
	
'	x+=ofx
'	y+=ofy
	line (x,y)-(x+1,y+1),col,bf
end sub

'' 
sub cydraw(ina as ubyte ptr,w as integer,h as integer)
	dim as integer i,j,x,y,a,b,c,d
	
	for j=0 to h-1
		for i=0 to (w/4)-1	
			a =(*ina and &h03)	
			b= (*ina and &h0c) shr 2
			c= (*ina and &h30) shr 4
			d= (*ina and &hc0) shr 6
			
			'pset (ofx+(i*4)+3,ofy+j),cycolor(a)
			'pset (ofx+(i*4)+2,ofy+j),cycolor(b)
			'pset (ofx+(i*4)+1,ofy+j),cycolor(c)
			'pset (ofx+(i*4),ofy+j),cycolor(d)
			
			block ((i shl 2)+3,j,cycolor(a)) '' shl 2 = *4
			block ((i shl 2)+2,j,cycolor(b))
			block ((i shl 2)+1,j,cycolor(c))
			block ((i shl 2),j,cycolor(d))	
			ina+=1
		next i
	next j

end sub

''
'' Main program
''

dim as string filename=""

filename = Command(1)

print TITLE+" ["+VERSION+"] by ssjx (2020) - "+SITEURL
	
if filename="" then
	print "Usage : cvc myfile.cvc"
	end 
else
	print "Loading file "+filename
end if
''


if  Open(filename For binary access read As #1)=0 then

	print "Loading File Header"
	get #1,,header
	
	if header.id<>"cyvd" then
		close(1)
		print("Aborting! This does not look like a valid CVC file!")
		print("Press a key to exit")
		sleep
		end
	end if
	
	print "Loading Video Header"
	get #1,,vid_header

	dim as integer cvc_width=swapshort(vid_header.x)
	dim as integer cvc_height=swapshort(vid_header.y)
	dim as integer cvc_fps=swapshort(vid_header.fps)
	dim as integer cvc_frames=swaplong(vid_header.frames)
	
	print "Size  : "+str(cvc_width)+" x "+str(cvc_height)
	print "Frames: "+str(cvc_frames)+" @ "+str(cvc_fps)+"fps"
	
	if cvc_fps=0 then 
		cvc_fps=10
		print "      : fps not set so using "+str(cvc_fps)+"fps"
	end if
	
	'' Some safety checks..
	dim as integer abort=0
	if cvc_width<0 or cvc_width>160 then abort=1
	if cvc_height<0 or cvc_height>100 then abort=1
	
	if abort=1 then
		close(1)
		print("Aborting! Dimensions are not correct!")
		print("Press a key to exit")
		sleep
		end
	end if
	
	'' If we are this far, CVC file is probably okay..
	dim frame as byte ptr
	dim display as byte ptr
	
	frame = Allocate(4000)
	display = Allocate(4000)

	if frame=0 or display=0 then
		close(1)
		print("Aborting! Could not allocated buffers!")
		print("Press a key to exit")
		sleep
		end
	end if

	memclr(display,4000)
	memclr(frame,4000)	'' Should never be >4000
	
	screen 14,32,1 '' 18
	windowtitle TITLE+" ["+VERSION+"] - "+SITEURL 
	
	'locate 27,0:
	'print"[R]eplay [P]ause [Q]uit" 
	
	dim as integer subtype,framesize
	
	for f as integer=0 to cvc_frames-1
		'print "Frame "+str(f)+" Header"
		get #1,,frm_header
	
		subtype=swapshort(frm_header.subtype)
		framesize=swapshort(frm_header.size)
		
		get #1,,*frame,framesize
		
		if framesize<=4000 then	
			select case as const subtype
			case 0:
			'	print "Type 0 Uncompressed ["+str(framesize)+"]"
				'' Don't assume frame is 4000 bytes!
				memcpy(display,frame,framesize)		
			case 1:
			'	print "Type 1 RLE ["+str(framesize)+"]"
				dec_enc_cv1(frame,display,framesize)
			case 2:
			'	print "Type 2 Delta ["+str(framesize)+"]"
				dec1_cv1(frame,display,framesize)	
			case 3:
			'	print "Type 3 Delta II ["+str(framesize)+"]"
				 dec2_cv1(frame,display,framesize)
			case else:
			'	print "Unknown frame type! ["+str(framesize)+"]"
			end select
		end if
		
		'' display
		screenlock
		cydraw(display,cvc_width,cvc_height)
		screenunlock
		
		if multikey(SC_ESCAPE) then
			exit for
		end if
	
		sleep (1000/cvc_fps)
	next f
	
	close(1)
	print "Done! Press Space to exit"
	sleep
	
	Deallocate(display)
	Deallocate(frame)

else
	print "Could not open video..."
end if


		
		
